
`timescale 1ns/1ps

`define C_S_AXI_DATA_WIDTH 32
`define C_S_AXI_ADDR_WIDTH 32

module master_axi_lite_iic
(
	S_AXI_ACLK,
     S_AXI_ARESETN,
     S_AXI_AWADDR,
     S_AXI_AWVALID,
     S_AXI_AWREADY,
     S_AXI_WDATA,
     S_AXI_WSTRB,
     S_AXI_WVALID,
     S_AXI_WREADY,
     S_AXI_BRESP,
     S_AXI_BVALID,
     S_AXI_BREADY,
     S_AXI_ARADDR,
     S_AXI_ARVALID,
     S_AXI_ARREADY,
     S_AXI_RDATA,
     S_AXI_RRESP,
     S_AXI_RVALID,
     S_AXI_RREADY,
     cstate_flag,
     error_flag
     
     //sled0   
);

 input                                 S_AXI_ACLK;
 input                                 S_AXI_ARESETN;
 output   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_AWADDR;
 output                                S_AXI_AWVALID;
 input                                 S_AXI_AWREADY;
 output   [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_WDATA;
 output   [`C_S_AXI_DATA_WIDTH/8-1:0]  S_AXI_WSTRB;
 output                                S_AXI_WVALID;
 input                                 S_AXI_WREADY;
 input  [1:0]                          S_AXI_BRESP;
 input                                 S_AXI_BVALID;
 output                                S_AXI_BREADY;
 output   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_ARADDR;
 output                                S_AXI_ARVALID;
 input                                 S_AXI_ARREADY;
 input  [`C_S_AXI_DATA_WIDTH-1:0]      S_AXI_RDATA;
 input  [1:0]                          S_AXI_RRESP;
 input                                 S_AXI_RVALID;
 output                                S_AXI_RREADY;
 output                                error_flag;
 output                                 cstate_flag;

 //output[6:0]							sled0;
 
//////////////////////////////////////////////////////////////////////////
//	Internal Wires/Registers
 reg   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_AWADDR;
 reg                                S_AXI_AWVALID;

 reg   [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_WDATA;
 reg   [`C_S_AXI_DATA_WIDTH/8-1:0]  S_AXI_WSTRB;
 reg                                S_AXI_WVALID;
 
 reg                                 S_AXI_BREADY;
 reg   [`C_S_AXI_ADDR_WIDTH - 1:0]   S_AXI_ARADDR;
 reg                                 S_AXI_ARVALID;

 reg                                 S_AXI_RREADY;

//define reg address of IIC-Master
  parameter 	PRERLO     = 5'b00000;
  parameter 	PRERHI     = 5'b00001;
  parameter 	CTR        = 5'b00010;
  parameter 	TXR        = 5'b00011; 
  parameter 	RXR        = 5'b00011; 
  parameter 	CR         = 5'b00100; 
  parameter 	SR         = 5'b00100;

  parameter     TX_DATA     = 8'b00000110;  //1e: 30
  
 
  parameter		CLK_Div_L	= 8'h63;  //
  parameter		CLK_Div_H	= 8'h00;  //

  parameter		EN_IP		=8'h80; 
  		
  parameter 	STA_WR_CR	=8'h90;	//start+write_ack
  parameter 	WR_CR		=8'h10; //write+ack
  parameter		STP_WR_CR	=8'h50; //stop+write+ack
  parameter		STP_CR		=8'h40; //stop+write+ack
   
  parameter 	STA_RD_CR	=8'hA0;//8'hC0;//start+read+ack     C0--->A0?
  parameter 	RD_CR		=8'h20;//read+ack
  parameter 	RD_NACKCR	=8'h28;//read+nack
  parameter 	STP_NCR	=8'h40;//stop+nack
  parameter 	STP_RD_NCR	=8'h68;//stop+read+nack
  
  parameter 	RD      	= 1'b1;
  parameter 	WR     		= 1'b0;
 
//define the address of IIC-Slave device 
  parameter 	DEV_ADDR	=7'b1010_000;
  
  reg			axi_awready_reg;
  reg			axi_arready_reg;
  reg			axi_rvalid_reg;
  
  reg[4:0]		wr_index/* synthesis syn_keep=1 */;
  reg[1:0]		wr_reg;
  reg[1:0]		rd_reg0;
  reg[1:0]		rd_ack;

  reg[1:0]		wr_rd_stop;
  
  reg[7:0]		reg_addr;
  reg[7:0]		reg_data;
  
  reg[31:0]		sr_data0;
  reg[31:0]		sr_ack;     //sr: status register?
  
//	reg[6:0]		sled0;
  //reg [7:0]     tx_data;
  reg           error_reg = 0;
  reg           cstate_flag_reg = 0;
//--------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////
always @(posedge S_AXI_ACLK)
begin
	if(~S_AXI_ARESETN)
		axi_awready_reg <= 1'b0;
	else
		axi_awready_reg <= S_AXI_AWREADY;
end

always @(posedge S_AXI_ACLK)
begin
	if(~S_AXI_ARESETN)
		axi_arready_reg <= 1'b0;
	else
		axi_arready_reg <= S_AXI_ARREADY;
end

always @(posedge S_AXI_ACLK)
begin
	if(~S_AXI_ARESETN)
		axi_rvalid_reg <= 1'b0;
	else
		axi_rvalid_reg <= S_AXI_RVALID;
end

//define the reg addr and data
always @(*)
begin
	if(~S_AXI_ARESETN)
	begin
		reg_addr <=0;
		reg_data <=0;

	end
	else
	begin
	case(wr_index)
	0:
	begin
		reg_addr <= PRERLO;
		reg_data <= CLK_Div_L;
	end
	1:
	begin
		reg_addr <= PRERHI;
		reg_data <= CLK_Div_H;
	end
	2:
	begin
		reg_addr <= CTR;
		reg_data <= EN_IP;
	end
	3:
	begin
		reg_addr <= TXR;
		reg_data <= {DEV_ADDR,WR};
	end
	4:
	begin
		reg_addr <= CR;
		reg_data <= STA_WR_CR;
	end
	5:
	begin
		reg_addr <= SR;
	end
	6:
	begin
		reg_addr <= TXR;
		reg_data <= 8'h00; //memory address High byte
	end
	7:
	begin
		reg_addr <= CR;
		reg_data <= WR_CR;
	end
	8:
	begin
		reg_addr <= SR;
	end
	9:
	begin
		reg_addr <= TXR;
		reg_data <= 8'h01; //8'h01-->8'h02 //memory address Low byte
	end
	10:
	begin
		reg_addr <= CR;
		reg_data <= WR_CR;
	end
	11:
	begin
		reg_addr <= SR;
	end
	12:
	begin
		reg_addr <= TXR;
		//reg_data <= 8'hf9; //write data, it will display "1" on the sled if correct
		//reg_data <= 8'h92; //write data, it will display "5" on the sled if correct
          reg_data <= TX_DATA;
	end
	13:
	begin
		reg_addr <= CR;
		reg_data <= STP_WR_CR;
	end
	14:
	begin
		reg_addr <= SR;
	end
	
	15:
	begin
		reg_addr <= TXR;
		reg_data <= {DEV_ADDR,WR};
	end
	16:
	begin
		
		reg_addr <= CR;
		reg_data <= STA_WR_CR;
	end	
	17:
	begin
		reg_addr <= SR;
	end
	
	18:
	begin
		reg_addr <= SR;
	end
	19:
	begin
		reg_addr <= TXR;
		reg_data <= 8'h00; //memory address High byte
	end
	20:
	begin
		reg_addr <= CR;
		reg_data <= WR_CR;
	end
	21:
	begin
		reg_addr <= SR;
	end
	22:
	begin
		reg_addr <= TXR;
		reg_data <= 8'h01; //memory address Low byte
	end
	23:
	begin
		reg_addr <= CR;
		reg_data <= WR_CR;
	end
	24:
	begin
		reg_addr <= SR;
	end
	25:
	begin
		reg_addr <= TXR;
		reg_data <= {DEV_ADDR,RD};
	end
	26:
	begin
		reg_addr <= CR;
		reg_data <= STA_WR_CR;
	end
	27:
	begin
		reg_addr <= SR;
	end
	28:
	begin
		reg_addr <= CR;
		reg_data <= RD_CR;
	end
	29:
	begin
		reg_addr <= SR;
	end
	30:
	begin
		reg_addr <= RXR;//read data from 01
	end
	31:
	begin
		reg_addr <= CR;
		reg_data <= STP_NCR;
	end	
	default:
	begin
		reg_addr <=0;
		reg_data <=0;
	end
	endcase
	end
end
		
		
//Firstly, vefiry the byte write mode of EEPROM
always @(posedge S_AXI_ACLK) 
begin
	if(~S_AXI_ARESETN)
	begin
		S_AXI_AWADDR <= {`C_S_AXI_ADDR_WIDTH{1'b0}};
		S_AXI_AWVALID <= 1'b0;
		S_AXI_WDATA <= {`C_S_AXI_DATA_WIDTH{1'b0}};
		S_AXI_WVALID <= 1'b0;
		S_AXI_WSTRB <= {`C_S_AXI_DATA_WIDTH/8 {1'b0}};
		
		S_AXI_ARVALID <= 1'b0;
		S_AXI_RREADY <= 1'b0;
		S_AXI_ARADDR <= {`C_S_AXI_ADDR_WIDTH{1'b0}};
		
		wr_index <= 0;
		wr_reg <= 0;
		
		rd_reg0 <=0;
		sr_data0 <=0;
		rd_ack <=0;	
		sr_ack <=0;
		wr_rd_stop <=0;

        //tx_data<=8'h08;
        error_reg<=0;
//		sled0 <= 0;
		
	end
	else 
	begin
		if((wr_index < 5) || ((wr_index >5) &&(wr_index < 8)) || ((wr_index >8) &&(wr_index < 11)) || 
		   ((wr_index >11) &&(wr_index < 14)) || ((wr_index >14) &&(wr_index < 17))||((wr_index >18) &&(wr_index < 21))||
		     ((wr_index>21)&&(wr_index<24))||((wr_index>24)&&(wr_index<27))||(wr_index==28)||(wr_index==31))
		begin
		case(wr_reg)
		0:
		begin
			S_AXI_AWVALID <= 1'b1;
			S_AXI_WVALID <= 1'b1;
			S_AXI_AWADDR <= {24'b0,reg_addr};
			S_AXI_WDATA  <= {24'b0,reg_data};
			S_AXI_WSTRB <= 4'b1111;
			
			wr_reg <= 1;
		end
		1:
		begin
			if(S_AXI_AWREADY && ~axi_awready_reg)
			begin
				S_AXI_AWVALID <= 1'b0;
				S_AXI_WVALID <= 1'b0;
				S_AXI_AWADDR <= 0;
				S_AXI_WDATA <= 0;
				S_AXI_WSTRB <= 0;
					
				wr_reg <= 2; 
			end
			else
			begin
				wr_reg <= 1;
			end
		end
		2:
		begin
			wr_index <= wr_index + 1;
			wr_reg <= 0;
		end
		endcase
		end
		else if((wr_index==5)||(wr_index==8)||(wr_index==11)||(wr_index==14)||(wr_index==17)||(wr_index==21)||
				  (wr_index==24)||(wr_index==27)||(wr_index==29))
			begin
			case(rd_reg0)
			0:
			begin
				S_AXI_ARVALID <= 1'b1;
				S_AXI_RREADY <= 1'b1;
				S_AXI_ARADDR <= {24'b0,reg_addr};
				
				rd_reg0 <= 1;
			end
			1:
			begin
				if(S_AXI_ARREADY && ~axi_arready_reg)
				begin
					S_AXI_ARVALID <= 1'b0;
					
					rd_reg0 <= 2; 
				end
				else
				begin
					rd_reg0 <= 1;
				end
			end
			2:
			begin
				if(S_AXI_RVALID && ~axi_rvalid_reg)
				begin
					S_AXI_RREADY <= 1'b0;
					sr_data0 <= S_AXI_RDATA;
					
					rd_reg0 <= 3;					
				end
				else
				begin
					rd_reg0 <= 2;
				end
			end	
			3:
			begin
					if(~sr_data0[1])
					begin
						wr_index <= wr_index + 1;
						rd_reg0 <= 0;
					end
					else
						rd_reg0 <= 0;
				end		
			endcase
			end	
			else if(wr_index==18)
			begin
			case(rd_ack)
			0:
			begin
				S_AXI_ARVALID <= 1'b1;
				S_AXI_RREADY <= 1'b1;
				S_AXI_ARADDR <= {24'b0,reg_addr};
				
				rd_ack <= 1;
			end
			1:
			begin
				if(S_AXI_ARREADY && ~axi_arready_reg)
				begin
					S_AXI_ARVALID <= 1'b0;
					
					rd_ack <= 2; 
				end
				else
				begin
					rd_ack <= 1;
				end
			end
			2:
			begin
				if(S_AXI_RVALID && ~axi_rvalid_reg)
				begin
					S_AXI_RREADY <= 1'b0;
					sr_ack <= S_AXI_RDATA;
					
					rd_ack <= 3;					
				end
				else
				begin
					rd_ack <= 2;
				end
			end	
			3:
			begin
					if(~sr_ack[7])
					begin
						wr_index <= wr_index + 1;
						rd_ack <= 0;
					end
					else
					begin
						wr_index <= 15;
						rd_ack <= 0;
					end
				end		
			endcase
			end	
			else if(wr_index==30)
			begin
			case(wr_rd_stop)
			0:
			begin
				S_AXI_ARVALID <= 1'b1;
				S_AXI_RREADY <= 1'b1;
				S_AXI_ARADDR <= {24'b0,reg_addr};
				
				wr_rd_stop <= 1;
			end
			1:
			begin
				if(S_AXI_ARREADY && ~axi_arready_reg)
				begin
					S_AXI_ARVALID <= 1'b0;
					
					wr_rd_stop <= 2; 
				end
				else
				begin
					wr_rd_stop <= 1;
				end
			end
			2:
			begin
				if(S_AXI_RVALID && ~axi_rvalid_reg)
				begin
                    cstate_flag_reg <= 1'b1;
					S_AXI_RREADY <= 1'b0;
//					sled0 <= S_AXI_RDATA[6:0];      //data from read register?
					wr_rd_stop <= 3;	
                    if(S_AXI_RDATA[7:0] != TX_DATA)
                        error_reg<=1; 
				end
				else
				begin
					wr_rd_stop <= 2;
				end
			end	
			3:
			begin
				wr_index <= wr_index + 1;
				wr_rd_stop <= 0;
			end		
			endcase
			end	
			
		end
	end

    assign error_flag=error_reg;
    assign cstate_flag = cstate_flag_reg;

endmodule					
